@@ -1,5 +1,3 @@
-from __future__ import unicode_literals
-
 import logging
 import random
 import threading
@@ -17,7 +15,7 @@ import mock
 
 import waffle
 from test_app import views
-from test_app.models import CompanyAwareFlag, Company
+from test_app.models import CompanyAwareFlag, CustomSample, CustomSwitch, Company
 from waffle.middleware import WaffleMiddleware
 from waffle.models import Sample, Switch
 from waffle.tests.base import TestCase
@@ -32,9 +30,13 @@ def get(**kw):
     return request
 
 
+def get_response(request):
+    return 'hello'
+
+
 def process_request(request, view):
     response = view(request)
-    return WaffleMiddleware().process_response(request, response)
+    return WaffleMiddleware(get_response).process_response(request, response)
 
 
 class WaffleTests(TestCase):
@@ -353,6 +355,22 @@ class WaffleTests(TestCase):
         assert not waffle.flag_is_active(request, 'foo')
         assert not hasattr(request, 'waffle_tests')
 
+    def test_testing_flag_header(self):
+        waffle.get_waffle_flag_model().objects.create(name='foo', testing=True)
+        request = RequestFactory().get('/foo', HTTP_DWFT_FOO='1')
+        request.user = AnonymousUser()
+        assert waffle.flag_is_active(request, 'foo')
+        assert 'foo' in request.waffle_tests
+        assert request.waffle_tests['foo']
+
+        # header should override cookie
+        request = RequestFactory().get('/foo', HTTP_DWFT_FOO='0')
+        request.user = AnonymousUser()
+        request.COOKIES['dwft_foo'] = 'True'
+        assert not waffle.flag_is_active(request, 'foo')
+        assert 'foo' in request.waffle_tests
+        assert not request.waffle_tests['foo']
+
     def test_set_then_unset_testing_flag(self):
         waffle.get_waffle_flag_model().objects.create(name='myflag', testing=True)
         response = self.client.get('/flag_in_view?dwft_myflag=1')
@@ -390,7 +408,7 @@ class WaffleTests(TestCase):
             self.assertEqual(b'on', response.content)
 
     @override_settings(WAFFLE_FLAG_MODEL='test_app.CompanyAwareFlag', AUTH_USER_MODEL='test_app.CompanyUser')
-    def test_pluggable_model(self):
+    def test_pluggable_flag_model(self):
         flag_model = waffle.get_waffle_flag_model()
         self.assertEqual(CompanyAwareFlag, flag_model)
 
@@ -520,6 +538,17 @@ class SwitchTests(TestCase):
             # the cache and DB are in sync.
             assert waffle.switch_is_active(switch.name)
 
+    @override_settings(WAFFLE_SWITCH_MODEL='test_app.CustomSwitch')
+    def test_pluggable_switch_model(self):
+        switch_model = waffle.get_waffle_model('SWITCH_MODEL')
+        self.assertEqual(CustomSwitch, switch_model)
+
+        switch_model.objects.create(name='test_switch_off', active=False)
+        switch_model.objects.create(name='test_switch_on', active=True)
+
+        assert not waffle.switch_is_active('test_switch_off')
+        assert waffle.switch_is_active('test_switch_on')
+
     @override_settings(WAFFLE_CREATE_MISSING_SWITCHES=True)
     @override_settings(WAFFLE_SWITCH_DEFAULT=False)
     def test_switch_created_dynamically_false(self):
@@ -614,6 +643,17 @@ class SampleTests(TestCase):
         waffle.sample_is_active('foo')
         mock_logger.log.assert_called_with(logging.WARNING, 'Sample %s not found', 'foo')
 
+    @override_settings(WAFFLE_SAMPLE_MODEL='test_app.CustomSample')
+    def test_pluggable_sample_model(self):
+        sample_model = waffle.get_waffle_model('SAMPLE_MODEL')
+        self.assertEqual(CustomSample, sample_model)
+
+        sample_model.objects.create(name='test_sample_off', percent=0)
+        sample_model.objects.create(name='test_sample_on', percent=100)
+
+        assert not waffle.sample_is_active('test_sample_off')
+        assert waffle.sample_is_active('test_sample_on')
+
 
 class TransactionTestMixin(object):
     """Mixin providing an abstract test case for writing in a transaction.
@@ -713,3 +753,26 @@ class SampleTransactionTests(TransactionTestMixin, TransactionTestCase):
 
     def toggle_is_active(self, sample):
         return waffle.sample_is_active(sample.name)
+
+
+class FunctionTests(TestCase):
+    @mock.patch.object(random, 'uniform')
+    def test_percent(self, uniform):
+        """If you have no cookie, you get a cookie!"""
+        uniform.return_value = '10'
+        waffle.get_waffle_flag_model().objects.create(name='myflag', percent='50.0')
+        request = get()
+        response = process_request(request, views.flag_in_view)
+        assert 'dwf_myflag' in response.cookies
+        self.assertEqual('True', response.cookies['dwf_myflag'].value)
+        self.assertEqual(b'on', response.content)
+
+    @mock.patch.object(random, 'uniform')
+    def test_percent_readonly(self, uniform):
+        uniform.return_value = '10'
+        """If you have no cookie, you do not get a cookie if we just have a question!"""
+        waffle.get_waffle_flag_model().objects.create(name='myflag', percent='50.0')
+        request = get()
+        response = process_request(request, views.flag_in_view_readonly)
+        assert 'dwf_myflag' not in response.cookies
+        self.assertEqual(b'off', response.content)
